"use strict";

let _ = require('underscore'),
    passport = require('passport'),
    LocalStrategy = require('passport-local').Strategy,
    FacebookTokenStrategy = require('passport-facebook-token').Strategy,
    GoogleTokenStrategy = require('passport-google-token').Strategy,
    models = require('mongoose').models,
    utilities = require('../utilities/index'),
    validator = require('../validator/index'),
    config = require('./config');



//// Public --------------------------------------------------------------------------------------


module.exports.init = function (server, options)  {
    if (options && _.isObject(options)) _.extend(config, options);

    server.use(passport.initialize());
    server.use(passport.session());

    passport.serializeUser      ( (user, done) => done(null, user) );
    passport.deserializeUser    ( (id, done) => done (null, id) );

    passport.use ( new LocalStrategy (
        function ( username, password, callback ) {
            utilities.logger.info('passport authenticating with localStrategy');
            let personArgs = validator.email(username) ? {email:username} : {username:username}
            models.Person.findOne (personArgs, (e, personExists) =>{
             
                if (e) { 
                    utilities.logger.error(e); 
                    return callback (new utilities.error.db (102));
                }
                
                if (!personExists) {
                    return callback (new utilities.error.app(402));
                }
                
                // if (!personExists.active) {
                //     return callback (new utilities.error.app(408));
                // }
             
                if (!personExists.auth.local || !personExists.auth.local.hash) {
                    return callback (new utilities.error.app(426));
                }
             
                // if (!personExists.confirmed) {
                //     return callback (new utilities.error.app(442));
                // }
                
                personExists.verifyLocalPassword (password, e => callback (e, personExists) );
            });
        }
    ));

    if (config.facebook) {
        passport.use(new FacebookTokenStrategy (
            {clientID: config.facebook.clientID, clientSecret: config.facebook.clientSecret},
            (accessToken, refreshToken, profile, callback) => {
                if (!profile){
                    return callback(new utilities.error.app (429));
                } else {
                    return callback(null, profile);
                }
            })
        );
    }

    if (config.google) {
        passport.use(new GoogleTokenStrategy (
            {clientID: config.google.clientID, clientSecret: config.google.clientSecret},        
            (accessToken, refreshToken, profile, callback) => {
                if (!profile) {
                    return callback(new utilities.error.app (471));
                } else {
                    return callback(null, profile);
                }
            })
        );
    }

};