process.env.testing = "true";

var assert      =   require("assert"),
    should      =   require('should'),
    crypto      =   require('../index');

crypto.init(require('../config'));

describe('#getRandomBytes()', function () {
    var randomPass1 = null;
    it('should get a random password salt (32 bytes)', function (done) {
        crypto.getRandomPasswordSalt(function (e, random) {
            should.not.exist(e); should.exist(random);
            random.should.match(/^[0-9a-f]+$/).and.have.length(64);
            randomPass1 = random;
            randomPass1.should.match(/^[0-9a-f]+$/).and.have.length(64);
            randomPass1.should.equal(random);
            done();
        });
    });

    it('should get a random password salt (32 bytes) synchronously', function (done) {
        crypto.getRandomPasswordSalt().should.match(/^[0-9a-f]+$/).and.have.length(64);
        done();
    });

    it('should get a random password salt (32 bytes)', function (done) {
        crypto.getRandomPasswordSalt(function (e, random) {
            should.not.exist(e); should.exist(random);
            random.should.match(/^[0-9a-f]+$/).and.have.length(64);
            random.should.not.equal(randomPass1);
            done();
        });
    });

    it('should get a random password reset code (16 bytes)', function (done) {
        crypto.getRandomPasswordReset(function (e, random) {
            should.not.exist(e); should.exist(random);
            random.should.match(/^[0-9a-f]+$/).and.have.length(32);
            done();
        });
    });

    it('should get a random password reset code (16 bytes) synchronously', function (done) {
        crypto.getRandomPasswordReset().should.match(/^[0-9a-f]+$/).and.have.length(32);
        done(); 
    });

    it('should get a random license code (12 bytes)', function (done) {
        crypto.getRandomLicenseCode(function (e, random) {
            should.not.exist(e); should.exist(random);
            random.should.match(/^[0-9a-f]+$/).and.have.length(24);
            done();
        });
    });

    it('should get a random license code (12 bytes) synchronously', function (done) {
        crypto.getRandomLicenseCode().should.match(/^[0-9a-f]+$/).and.have.length(24);
        done();
    });

    it('should get a random facebook token (32 bytes)', function (done) {
        crypto.getRandomFacebookToken(function (e, random) {
            should.not.exist(e); should.exist(random);
            random.should.match(/^[0-9a-f]+$/).and.have.length(64);
            done();
        });
    });

    it('should get a random facebook token (32 bytes) synchronously', function (done) {
        crypto.getRandomFacebookToken().should.match(/^[0-9a-f]+$/).and.have.length(64);
        done();
    });
});


describe('#hash()', function () {
    var hashArgs = { cipher: "sha256", encodingIn: "utf8", encodingOut: "hex", payload: "" };
    
    it('should test against a vector', function (done) {
        hashArgs.payload = "abc";
        var digest = crypto.hash(hashArgs);
        should.exist(digest);
        digest.should.match(/^[0-9a-f]+$/).and.have.length(64);
        digest.should.equal("ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad");
        done();
    });
    
    it('should test against a vector', function (done) {
        hashArgs.payload = "message digest";
        var digest = crypto.hash(hashArgs);
        should.exist(digest);
        digest.should.match(/^[0-9a-f]+$/).and.have.length(64);
        digest.should.equal("f7846f55cf23e14eebeab5b4e1550cad5b509e3348fbc4efa3a1413d393cb650");
        done();
    });

    it('should test against a vector', function (done) {
        hashArgs.payload = "secure hash algorithm";
        var digest = crypto.hash(hashArgs);
        should.exist(digest);
        digest.should.match(/^[0-9a-f]+$/).and.have.length(64);
        digest.should.equal("f30ceb2bb2829e79e4ca9753d35a8ecc00262d164cc077080295381cbd643f0d");
        done();
    });

    it('should test against a vector', function (done) {
        hashArgs.payload = "SHA256 is considered to be safe";
        var digest = crypto.hash(hashArgs);
        should.exist(digest);
        digest.should.match(/^[0-9a-f]+$/).and.have.length(64);
        digest.should.equal("6819d915c73f4d1e77e4e1b52d1fa0f9cf9beaead3939f15874bd988e2a23630");
        done();
    });

    it('should test against a vector', function (done) {
        hashArgs.payload = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq";
        var digest = crypto.hash(hashArgs);
        should.exist(digest);
        digest.should.match(/^[0-9a-f]+$/).and.have.length(64);
        digest.should.equal("248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1");
        done();
    });

    it('should test against a vector', function (done) {
        hashArgs.payload = "For this sample, this 63-byte string will be used as input data";
        var digest = crypto.hash(hashArgs);
        should.exist(digest);
        digest.should.match(/^[0-9a-f]+$/).and.have.length(64);
        digest.should.equal("f08a78cbbaee082b052ae0708f32fa1e50c5c421aa772ba5dbb406a2ea6be342");
        done();
    });

    it('should test against a vector', function (done) {
        hashArgs.payload = "This is exactly 64 bytes long, not counting the terminating byte";
        var digest = crypto.hash(hashArgs);
        should.exist(digest);
        digest.should.match(/^[0-9a-f]+$/).and.have.length(64);
        digest.should.equal("ab64eff7e88e2e46165e29f2bce41826bd4c7b3552f6b382a9e7d3af47c245f8");
        done();
    });
});

describe('#hashPassword()', function () {
    var hash1 = null
    it('should hash a value with a salt', function (done) {
        hashPassArgs = {value: 'Hilton39' , salt: 'Hilton39' }
        var digest = crypto.hashPassword(hashPassArgs);
        should.exist(digest);
        digest.should.match(/^[0-9a-f]+$/).and.have.length(64);
        hash1 = digest;
        done();
    });

    it('should hash a value with a salt', function (done) {
        hashPassArgs = {value: 'Hilton39' , salt: 'Hilton39' }
        var digest = crypto.hashPassword(hashPassArgs);
        should.exist(digest);
        digest.should.match(/^[0-9a-f]+$/).and.have.length(64);
        digest.should.equal(hash1);
        done();
    });
});

describe('#createPasswordHash()', function () {
    var hash1 = null, salt1 = null;
    
    it('should generate salt, create a hash and return hash + salt', function (done) {
        crypto.createHashPassword({ value: 'Hilton39' }, function (e, data) {
            should.exist(data);
            should.exist (data.hash);
            should.exist (data.salt);
            data.hash.should.match(/^[0-9a-f]+$/).and.have.length(64);
            hash1 = data.hash;
            data.salt.should.match(/^[0-9a-f]+$/).and.have.length(64);
            salt1 = data.salt;
            done();
        });
    });

    it('should generate salt, create a hash and return hash + salt synchronously', function (done) {
        var data = crypto.createHashPassword({ value: 'Hilton39' });
        should.exist(data);
        should.exist (data.hash);
        should.exist (data.salt);
        data.hash.should.match(/^[0-9a-f]+$/).and.have.length(64);
        data.hash.should.not.equal(hash1)
        data.salt.should.match(/^[0-9a-f]+$/).and.have.length(64);
        data.salt.should.not.equal(salt1)
        done();
    });
});
