"use strict";
let Promise 				=	require ('bluebird'),
	_						= 	require ('underscore'),
	mongoose 				= 	require ('mongoose'),
	events 					=	require ('../events/index'),
	utilities   			=   require ('../utilities/index'),
	config					= 	require ('./config'),
	db 						= 	mongoose.connection,
	env 					= 	process.env.NODE_ENV || 'development',
	models 					= 	mongoose.models,
	Schema 					= 	mongoose.Schema,
	alive 					= 	false;


mongoose.Promise = require('bluebird');

//// Public --------------------------------------------------------------------------------------


exports.init = function (options) {

	return new Promise(function (resolve, reject) {

		if (options && _.isObject(options)) _.extend (config, options);

		if ( !db.readyState && db.readyState !== 1 && db.readyState !== 2 ) {

			utilities.logger.info ('Connecting to: ' + config.uri);

			mongoose.connect ( config.uri, config.dbOptions );

			db.on ('error', function ( err ) {
				utilities.logger.error ('Error connecting to database.');
				utilities.logger.error (err);
				if (!alive) mongoose.disconnect (reject);
			});

			db.on ('connected', () => utilities.logger.info ('Connected to database.') );
			db.on ('disconnected', () => utilities.logger.info ('Disconnected from database.') );

			db.once ('open', function ( ) {
				utilities.logger.info ('Database successfully opened.');
				alive = true;

				if ( _.isEmpty ( mongoose.models ) ) _.each ( config.models, function (model, modelName) {
						let schema = new Schema(model.model, model.options ? model.options : config.defaultSchemaOptions);
						if (model.methods) schema.methods = model.methods;
						mongoose.model ( modelName, schema );
					});
				events.emit('wsf-databaseconnected');
				return resolve (null);
			});
		} else return resolve (null);

    });



}

exports.getBulkOperationHandler = function (collectionName) {
	var col = db.collection(collectionName);
    var bulk = col.initializeUnorderedBulkOp();
    return bulk;
}