"use strict";

let Schema 			= 		require('mongoose').Schema;

exports.model = {
	active: 		{ type: Boolean, default: true},
	auth: 			{
						local: {
							hash: 	String,
							salt: 	String,
							reset: 	{ code: {type: String}, time: {type: Date, default: Date.now} }
						},
						facebook: 	{ id: String, hash: String },
						twitter: 	{},
						google: 	{},
						openID: 	{}
					},
	avatar: 		{ url: String, fileType: String, bytes: Number },
	email: 			{ type: String, lowercase: true, trim: true, required: true },
	location: 		[ {
						locType: 	{ type: String, trim: true},
						address1: 	{ type: String, trim: true},
						address2: 	{ type: String, trim: true},
						city: 		{ type: String, trim: true},
						state: 		{ type: String, trim: true},
						postal: 	{ type: String, trim: true},
						country: 	{ type: String, trim: true}
					} ],
	name: 			{ type: String, trim: true },
	phone: 			[ {
						phType: 	{ type: String, trim: true},
						areaCode: 	{ type: String, lowercase: true, trim: true },
						number: 	{ type: String, lowercase: true, trim: true },
						ext: 		{ type: String, lowercase: true, trim: true },
					} ],
	notifications: 	[ {  _id:false, platform: String, token: String, targetArn: String  } ],
	time: 			{
						created: 	{ type: Date, default: Date.now },
						deactivated: Date
					},
	username: 		{ type: String, trim: true },
	verified: 		Boolean

};

exports.methods = {
	isOwner: function ( personId ) {
	    return this._owner.equals(personId);
	}
};


exports.methods = {
	deactivate: function () {
	    if (this.active) {
	    	this.set({
	    		active: false,
	    		notifications: [],
	    		deactivated: Date.now()
	    	});
	    }
	    return this;
	},
	setLocalPassword: function ( password, callback ) {
	  	utilities.logger.info('bin/models/person setLocalPassword');
	  	let person = this;
	    crypto.createHashPassword( password, function (e, data) {
	        if (e) return callback(e);
	        //_.extend (person.auth.local, data );
	        person.auth.local = data
	        return callback (null, person);
	    });
	},
	setFacebookToken: function ( id, callback ) {
	  	utilities.logger.info('bin/models/person setFacebookToken');
	  	let person = this;
	    crypto.getRandomFacebookToken( function (e, data) {
	        if (e) return callback(e);
	        if (!person.auth.facebook) person.auth.facebook = { id: id, hash: data };
	        else _.extend (person.auth.facebook, { id: id, hash: data } );
	        return callback (null, person);
	    });
	},
	verifyLocalPassword: function ( password, callback ) {
	  	utilities.logger.info('bin/models/person verifyLocalPassword');
	  	let person = this,
	  		digest = crypto.hashPassword( { value:password, salt:person.auth.local.salt });
	    if (callback) return callback (digest === person.auth.local.hash ? null :  new utilities.error.app(402)); //async
	    return digest === person.auth.local.hash; //sync
	}
}