process.env.testing = "true";

var assert      =   require("assert"),
	crypto      =   require('../../crypto/index'),
    database 	= 	require('../index'),
	models 		= 	database.models,
    should      =   require('should'),
    person1 	= 	{
						name: "Ahsoka Tano",
						username: "atano",
						email: "ahsoktano001@gmail.com",
					}, 
	person 		= 	null;

crypto.init(require('../../crypto/config'));

describe('#database', function () { 
	it('should check that the database was opened', function (done) {
		database.init( require('../config'), function (e){ should.not.exist(e); done(); });
	});
});

describe('#modelPerson', function () { 
	var hash1 = null, salt1 = null;
	it('should check that the new model was instantiated', function (done) {
        person = new models.Person (person1);
        should.exist (person);
        should.exist (person.name); person.name.should.equal(person1.name);
        done();
    });

	it('should generate a hash and salt given a password', function (done) {
        person.setLocalPassword ( "Hilton39", function (e, person) {
            should.not.exist(e); 
            should.exist (person);
            should.exist (person.auth.local);
            should.exist (person.auth.local.hash); hash1 = person.auth.local.hash;
            should.exist (person.auth.local.salt); salt1 = person.auth.local.salt;
            done();
        });
    });

    it('should generate a new hash and salt given the same password', function (done) {
        person.setLocalPassword ( "Hilton39", function (e, person) {
            should.not.exist(e); 
            should.exist (person.auth.local);
            should.exist (person.auth.local.hash); person.auth.local.hash.should.not.equal(hash1);
            should.exist (person.auth.local.salt); person.auth.local.salt.should.not.equal(salt1);
            done();
        });
    });

	it('should verify the current password', function (done) {
        person.verifyLocalPassword ( "Hilton39").should.equal(true);
        done();
    });

    it('should not verify a wrong password', function (done) {
        person.verifyLocalPassword ( "Hilton40").should.equal(false);
        done();
    });

    it('should verify the current password asynchronously', function (done) {
        person.verifyLocalPassword ( "Hilton39", function (e, person) {
            should.not.exist(e); 
            done();
        });
    });

    it('should not verify a wrong password asynchronously', function (done) {
        person.verifyLocalPassword ( "Hilton40", function (e, person) {
            should.exist(e); e.code.should.equal(402);
            done();
        });
    });
});

		