var _ = 		require('underscore'),
	util =      require('util'),
	config = 	require('./config'),
	logger =    require('../logger/index'),

	AbstractWarn = function (errType, code, message, constr) {
		Error.captureStackTrace(this, constr || this);
		this.message = message ? `${message} (${code})` : `${config.messages['EN'][errType][code]} (${code})`;
		this.code = code;
	},

	AbstractError = function (errType, code, message, constr) {
		Error.captureStackTrace(this, constr || this);
		this.message = message ? `${message} (${code})` : `${config.messages['EN'][errType][code]} (${code})`;
		this.code = code;
		logger.error (this.stack);
	};

util.inherits(AbstractError, Error)
AbstractError.prototype.name = 'Abstract Error'

util.inherits(AbstractWarn, Error)
AbstractWarn.prototype.name = 'Abstract Warn'



var ApplicationError = function (code, message) {ApplicationError.super_.call(this, 'app', code, message, this.constructor) };
util.inherits(ApplicationError, AbstractWarn);
ApplicationError.prototype.name = 'Application Error';

var DatabaseError = function (code) {DatabaseError.super_.call(this, 'db', code, this.constructor) };
util.inherits(DatabaseError, AbstractError);
DatabaseError.prototype.name = 'Database Error';

var ServerError = function (code) {ServerError.super_.call(this, 'srv', code, this.constructor) };
util.inherits(ServerError, AbstractError);
ServerError.prototype.name = 'Server Error';






var AWSError = function (code, message) {AWSError.super_.call(this, 'aws', config.awsErrorTranslate[code] || 800, message, this.constructor) };
util.inherits(AWSError, AbstractWarn);
AWSError.prototype.name = 'AWS Error';

var FBError = function (code, message) {FBError.super_.call(this, 'fb', config.fbErrorTranslate[code] || 950, message, this.constructor) };
util.inherits(FBError, AbstractWarn);
FBError.prototype.name = 'FB Error';

var GOOGError = function (code, message) {GOOGError.super_.call(this, 'goog', config.googErrorTranslate[code] || 1050, message, this.constructor) };
util.inherits(GOOGError, AbstractWarn);
GOOGError.prototype.name = 'FB Error';

var PaypalError = function (code, message) {PaypalError.super_.call(this, 'ppl', code || 1000, message, this.constructor) };
util.inherits(PaypalError, AbstractWarn);
PaypalError.prototype.name = 'Paypal Error';

var StripeError = function (code, message) {StripeError.super_.call(this, 'stp', config.stpErrorTranslate[code] || 1200, message, this.constructor) };
util.inherits(StripeError, AbstractWarn);
StripeError.prototype.name = 'Stripe Error';

module.exports = {
	db: 	DatabaseError,
	srv: 	ServerError,
	app: 	ApplicationError,
	aws: 	AWSError,
	fb: 	FBError,
	goog: 	GOOGError,
	ppl: 	PaypalError,
	stp: 	StripeError,
	init: 	options => { if (options && _.isObject(options)) _.extend(config, options); }
}