/**
 * Created by andy.batchelor on 2/26/14.
 */

"use strict";

let _ =             require('underscore'),
    nodeSlack =     require('node-slack'),
    util =          require('util'),
    winston =       require('winston');

const defaultFormatter = (level, msg, meta) => {
    let message = "",
        metadata = "";

    if (msg) {
        if (_.isObject(msg)) {
            if (msg.stack) {
                message = JSON.stringify(msg.stack);
            } else {
                message = JSON.stringify(msg);
            }
        } else {
            message = msg
        }
    }

    if (meta) {
        if (_.isObject(meta)) {
            if (meta.stack) {
                metadata = JSON.stringify(meta.stack);
            } else {
                metadata = JSON.stringify(meta);
            }
        } else {
            metadata = meta
        }
    }

    return { text: "[" + level + "] " + message + " || " + metadata };
}

let Slack = exports.Slack  = function (options) {
    options = options || {};
    //if(!options.webhook_url || !options.domain){
    if(!options.webhook_url){
        throw new Error("options cannot be null");
    } else {
        this.webhook_url   = options.webhook_url;
        this.channel    = options.channel;
        //this.domain     = options.domain;
        this.username   = options.username || "winston-slack";
        this.level      = options.level    || 'info';
        this.silent     = options.silent   || false;
        this.raw        = options.raw      || false;
        this.name       = options.name     || 'slack';
        this.formatter  = options.customFormatter || defaultFormatter;

        //- Enabled loging of uncaught exceptions
        this.handleExceptions = options.handleExceptions || false

        //- Configure node-slack module
        this.slack = new nodeSlack(this.webhook_url);
    }
};

//
// Inherit from `winston.Transport` so you can take advantage
// of the base functionality and `.handleExceptions()`.
//
util.inherits(Slack, winston.Transport);

//- Attaches this Transport to the list of available transports
winston.transports.Slack = Slack;


Slack.prototype.log = function (level, msg, meta, callback) {
    //- Use custom formatter for message if set
    let message = this.formatter(level, msg, meta);
    /*let message = this.formatter
        ? this.formatter(level, msg, meta)
        : { text: "[" + level + "] " + (_.isString (msg) ? msg : JSON.stringify(msg) ) + (_.isString(meta) ? meta : JSON.stringify(meta) ),
            channel: this.channel,
            username: this.username
        };*/
    this.slack.send(message);

    callback(null, true);
};