"use strict";

let fs                  =   require('fs'),
    _                   =   require('underscore'),
    winston             =   require('winston'),
    slack               =   require('./bin/slack').Slack,

    env                 =   process.env.NODE_ENV,
    config              =   require('./config'),

    noop                =   dummy => {};


// remove default console logger
winston.remove(winston.transports.Console);

_.each(config.loggerLevels.levels, (levelNumber, levelName) => exports[levelName] = noop );

function setExports (level) {
    if ( level && _.has(config.loggerLevels.levels, level) && _.has(winston, level) ) {
        let highestLevel = config.loggerLevels.levels[level];
        _.each(config.loggerLevels.levels, function (levelNumber, levelName) {
            if (levelNumber <= highestLevel) exports[levelName] = winston[levelName];
        });
    }
};

//// Public ---------------------------------------------------------------------------------------
exports.init = function (options)  {
    if (options && _.isObject(options)) _.extend(config, options);

    // reset file content
    if (config && config.file && config.file[env] && config.file[env].filename && config.file[env].enabled) fs.closeSync ( fs.openSync(config.file[env].filename, 'w') ); //truncate file

    if (config.console && config.console[env] && config.console[env].enabled) {
        winston.add( winston.transports.Console, _.omit (config.console[env], 'enabled'));
        setExports (config.console[env].level);
    }
    if (config.file && config.file[env] && config.file[env].enabled) {
        winston.add( winston.transports.File, _.omit (config.file[env], 'enabled'));
        setExports (config.file[env].level);
    }
    if (config.slack && config.slack[env] && config.slack[env].enabled) {
        winston.add( slack, _.omit (config.slack[env], 'enabled') || {});
        setExports (config.slack[env].level);
    }

};