"use strict";

let _               =   require('underscore'),
    mailer          =   require("nodemailer"),
    htmlToText      =   require('html-to-text'),
    // htmlToText      =   require('nodemailer-html-to-text').htmlToText,

    config          =   {},
    utilities       =   require('../utilities/index'),
    transports      =   {},
    htmlToTextOptions = {
                            ignoreImage:                true,
                            hideLinkHrefIfSameAsText:   true
                        };



function isEmailValid (data) {
    return true;
    /*let emailDomain = data.to.split('@')[1];
    if (emailDomain === 'facebook.com' || emailDomain === 'tfbnw.net') {
        utilities.logger.warn (data.to + " is not a proper email");
        return false;
    } else return true;*/
}


//// Public --------------------------------------------------------------------------------------

exports.init = function (options)  {
    if (options && _.isObject(options)) _.extend(config, options);

    // Set up transports
    _.each (config.transports, function (transport, name){
        if (transport.enabled) {
            transports[name] = mailer.createTransport(transport.transportObject(transport.options));
            // transports[name].use('compile', htmlToText);
        }
        else transports[name] = false;
    });

    if (process.env.NODE_ENV === 'development') {
        if (!config.enabled.dev) exports.send = (dummy, callback) => callback (null, {});
    }

    if (process.env.NODE_ENV === 'staging') {
        if (!config.enabled.stg) exports.send = (dummy, callback) => callback (null, {});
    }

    if (process.env.NODE_ENV === 'production') {
        if (!config.enabled.prod) exports.send = (dummy, callback) => callback (null, {});
    }

};

exports.send = function (data, callback) {

    utilities.logger.info('framework/mailer send');

    if (!data.to) return callback (null);

    let mailType = config.types[data.mailType];

    if (!mailType) return callback (-1);

    if (!isEmailValid) return callback (null);

    if (!data.locale || !config.strings[data.locale]) data.locale = 'en';
    else data.locale = data.locale.toLowerCase();

    let templateArgs = mailType.setArgs(data);

    _.extend (templateArgs, data);

    let options = {
            to:         data.to,
            cc:         data.cc || "",
            bcc:        data.bcc || "",
            subject:    mailType.template.subject(templateArgs),
            //text:       mailType.template.text(templateArgs),
            html:       mailType.template.html(templateArgs)
        },
        mailOptions = _.extend(config.defaults, options);

    mailOptions.text =  htmlToText.fromString(mailOptions.html, {ignoreImage: true});

    transports.SES.sendMail ( mailOptions, function (e, response) {
        if (e) {
            let err = new utilities.error.aws (e.code);
            utilities.logger.error(err);
            utilities.logger.error(e);
            return callback ( err );
        }
        return callback(null);
    });
};

exports.sendTest = function (data, callback) {

    utilities.logger.info('framework/mailer sendTest');

    let options = {
            to:         data.email,
            cc:         data.cc || "",
            bcc:        data.bcc || "",
            subject:    "Bunz Test Email",
            //text:       mailType.template.text(templateArgs),
            html:       data.emailTemplate
        },
        mailOptions = _.extend(config.defaults, options);

    mailOptions.text =  htmlToText.fromString(mailOptions.html);

    transports.SES.sendMail ( mailOptions, function (e, response) {
        if (e) {
            let err = new utilities.error.aws (e.code);
            utilities.logger.error(err);
            utilities.logger.error(e);
            return callback ( err );
        }
        return callback(null);
    });
};








