const logger = require('../logger/index');
const pubsubLib = require('@google-cloud/pubsub');

const pubsub = initPubSub();

function wrapInTry(func){
  let result;
  try {
    result = func();
  } catch(e) {
    logger.error(e);
  } finally {
    return result;
  }
}

//incase credentials to pubsub are wrong, we do not want to crash the server
function initPubSub(){
  return wrapInTry( () =>
    pubsubLib(getObjFromBase64(process.env.GOOGLE_CREDENTIALS_B64))
  );
}

function getObjFromBase64(str) {
  const getAscii = () => new Buffer(str, 'base64').toString('ascii');
  return JSON.parse(getAscii(str));
}

function buildPayload(dataObj, results) {
  const { args, req } = dataObj.data;
  return {
    url: req.route.path,
    args: args,
    session: req.session,
    header: req.headers,
    eventTime: Date.now(),
    results: results,
  };
}
// publish event, if topic doesn't exist (404) create topic, create topic!
function publishIfExist(topic, payload) {
  topic.publish(payload, (err) => {
    if(err && err.code === 404){
      topic.get({ autoCreate: true }, (err, topic) => {
        topic.publish(payload)
      });
    }
  });
}

function publish(data, result) {
  const dataObj = {data};
  if(pubsub){
    const topic = pubsub.topic('event');
    const payload = buildPayload(dataObj, result);
    wrapInTry(() => {
      publishIfExist(topic, payload);
    }
    );
  }
};

module.exports = {
  publish,
};

