"use strict";

let _               =   require('underscore'),
	AWS             =   require('aws-sdk'),
	models          =   require ('mongoose').models,
	sqs             =   null,
	utilities       =   require('../utilities/index'),
	interfaces      =   require('../interfaces/index'),
	config          =   {},
	logErrorAndReturn =     function (e, result, callback) {
                                let err = null;
                                if (e) {
                                    err = new utilities.error.aws (e.code);
                                    utilities.logger.error(e.stack);
                                }
                                if (callback) return callback (err, result);
                            };



//// Public --------------------------------------------------------------------------------------

exports.init = function (options)  { 
	if (options && _.isObject(options)) _.extend(config, options);
	AWS.config.update (_.pick(config, 'accessKeyId', 'secretAccessKey'));
	sqs = new AWS.SQS({region: config.region});
};

exports.sendMessage = function (data) { //data = {  }
	utilities.logger.info('framework/queue sendMessage');
	
	let params = null;

	if (config.paramBuilder [data.msgType]) params = config.paramBuilder[data.msgType](data);
	else {
		utilities.logger.error("No handler for message type " + data.msgType);
		return;
	}
	
	sqs.sendMessage(params, function(e, data) {
		if (e) {
			utilities.logger.error(e);
			utilities.logger.error(e.stack);
		}
	});

};

exports.receiveMessage = function (data, callback) { //data = {  }
	utilities.logger.info('framework/queue receiveMessage');

	var params = {
		QueueUrl: config.platform.url,
		/*AttributeNames: [
			'Policy | VisibilityTimeout | MaximumMessageSize | MessageRetentionPeriod | ApproximateNumberOfMessages | ApproximateNumberOfMessagesNotVisible | CreatedTimestamp | LastModifiedTimestamp | QueueArn | ApproximateNumberOfMessagesDelayed | DelaySeconds | ReceiveMessageWaitTimeSeconds | RedrivePolicy',
		],*/
		MaxNumberOfMessages: 10,
		MessageAttributeNames: ['All'],
		VisibilityTimeout: 5,
		//VisibilityTimeout: 1800,
		//WaitTimeSeconds: 0
	};

/*	sqs.receiveMessage(params, function(err, data) {
		if (e) {
			utilities.logger.error(e);
			utilities.logger.error(e.stack);
		} else {
			return callback (null)
		}
	});*/

	sqs.receiveMessage ( params, (e, result) => logErrorAndReturn (e, result, callback) );

};



if (process.env.testing === 'true') {    
 
} else {
  
}

// Message Template
/*var params = {
		MessageBody: 'STRING_VALUE', // required
		QueueUrl: 'STRING_VALUE', //required
		DelaySeconds: 0,
		MessageAttributes: {
			someKey: {
				DataType: 'STRING_VALUE', //required
				
				BinaryListValues: [
					new Buffer('...') || 'STRING_VALUE',
					// more items
				],
				
				BinaryValue: new Buffer('...') || 'STRING_VALUE',
				
				StringListValues: [
					'STRING_VALUE',
					// more items
				],
				
				StringValue: 'STRING_VALUE'
			},
		}
	};*/