"use strict";

let Promise         =   require('bluebird'),
    _               =   require('underscore'),
    AWS             =   require('aws-sdk'),
    s3              =   require ('s3'),
    fs              =   require ('fs'),
    models          =   require ('mongoose').models,
    crypto          =   require('../crypto/index'),
    utilities       =   require('../utilities/index'),
    s3West          =   new AWS.S3({ accessKeyId: process.env.AWS_ACCESS_KEY_ID, secretAccessKey: process.env.AWS_SECRET_KEY, region: "us-west-2" }),
    env             =   process.env.NODE_ENV || 'development',
    s3Client        =   null,
    config          =   {};

function reverse (s) {
    let o = '';
    for (let i = s.length - 1; i >= 0; i--) { o += s[i] };
    return o;
}

function getUniqueKey ( id ) {
    let preHash = Date.now() + id,
        folder = (reverse(id)).substr(0,6) + '/',
        filename =  crypto.createHashFilename(preHash);
    return folder + ( (filename && filename.hash) ? filename.hash : preHash );
}

function deleteLocalFile (fullPath) {
    fs.unlink (fullPath, function (e) {
        if (e) {
            utilities.logger.error(e);
            new utilities.error.srv(203);
        }
        else utilities.logger.info ('Deleted: ' + fullPath);
    });
}

//// Public --------------------------------------------------------------------------------------

module.exports.init = options => {
    utilities.logger.info('framework/notify/bin/AWSS3 init');
    if (options) _.extend (config, options);
    if (!s3Client) s3Client = s3.createClient (config.options);
    utilities.logger.info(JSON.stringify(config.options));
};

/*module.exports.store = function (data) { //data = { itemType: String, localFile: String, _id: String, region: String, key: String (optional) }
    utilities.logger.info('framework/storage store');
    return new Promise ((resolve, reject) => {
        imageResize(data)
        .then( results => {
            let item = config.itemTypes[data.itemType],
                params = {
                    localFile: data.localFile,
                    s3Params: { Bucket: item.bucket, Key: data.key || getUniqueKey(data._id), ACL: item.acl },
                    // other options supported by putObject, except Body and ContentLength.
                    // See: http://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/S3.html#putObject-property
                };

            let uploader = s3Client.uploadFile(params);

            uploader.on('error',    e  => { utilities.logger.error (e); deleteLocalFile(data.localFile); });
            uploader.on('end',      () => { utilities.logger.info('**** Upload completed ****'); deleteLocalFile(data.localFile); } );

            if (env !== 'production' ) uploader.on('progress', () => utilities.logger.info ("File upload Progress: " + uploader.progressMd5Amount + ' | Amount: ' + uploader.progressAmount + ' | Total: ' + uploader.progressTotal) );

            // return without waiting for upload to complete, assuming everything went ok. This is because s3 can be slow as molasses sometimes and we want to avoid a 504 timeout error.
            return resolve ({
                vendor: { id: 'awss3', bucket: params.s3Params.Bucket, key: params.s3Params.Key },
                url: (item.secure ? 'https://' : 'http://') +  params.s3Params.Bucket + '.s3.amazonaws.com/' + params.s3Params.Key
            });
        })
        .catch (reject);
    });
};*/

module.exports.getSignedURL = function (data) { //data = { itemType: String, localFile: String, _id: String, region: String }
    utilities.logger.info('framework/storage getSignedURL');


    return new Promise ((resolve, reject) => {

        const item = config.itemTypes[data.itemType],

            s3_params = {
                Bucket: item.bucketWest,
                Key: data.key || getUniqueKey(data._id),
                Expires: 1200,
                ContentType: data.fileType || "",
                ACL: 'public-read',
            };

        s3West.getSignedUrl ('putObject', s3_params, (e, data) => {
            if (e) {
                let err = new utilities.error.aws (e.code);
                utilities.logger.error(err);
                utilities.logger.error(e);
                reject(err);
                return null;
            } else {
                resolve({
                    signedRequest: data,
                    vendor: { id: 'awss3', bucket: s3_params.Bucket, key: s3_params.Key },
                    url: (item.secure ? 'https://' : 'http://') + config.hostWest + '/' + s3_params.Bucket + '/' + s3_params.Key
                });
                return null;
            }
        });
    });
};

module.exports.delete = function (data) { //data = { id: String, bucket:String, key: String }
    utilities.logger.info('framework/storage delete');

    return new Promise ((resolve, reject) => {

        let item = config.itemTypes [data.itemType],
            params = {
                Bucket: data.bucket,
                Delete: {  Objects: [ { Key: data.key }, { Key: data.key + "x200" } ], }
            };

        switch (data.bucket) {
            case "avatars.prod.us.shufl.it":
            case "avatars.dev.us.shufl.it":
            case "itemimages.prod.us.shufl.it":
            case "itemimages.dev.us.shufl.it":
            case "itemvideos.prod.us.shufl.it":
            case "itemvideos.dev.us.shufl.it":
            case "messageimages.prod.us.shufl.it":
            case "messageimages.dev.us.shufl.it":
                let eraser = s3Client.deleteObjects(params);
                eraser.on('error', e => {
                    let err = new utilities.error.aws (e.code);
                    utilities.logger.error(err);
                    utilities.logger.error(e);
                    return reject (err);
                });
                return eraser.on('end', resolve );
                break;
            default:
                return s3West.deleteObjects(params).promise()
                    .then(resolve)
                    .catch(reject);
        }

    });
};
