"use strict";
let _ 				= 	require('underscore'),
    models          =   require('mongoose').models,
    Promise         =   require ('bluebird'),
    logger          =   require('../logger/index'),
    error           =   require('../error/index'),
    http            =   require('./bin/http'),
    transact        =   require('./bin/transact'),
    database        =   require('./bin/database'),
    escapeRegEx     =   {
                            allSpecial:                 /[-[\]{}()*+?.,"'&!:~\\^$|#\s.]/g,
                            blackSlashAndApostrophe:    /[\'|\\]/g,
                            aws:                        /[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|\!\&\"\~\:]/g,
                            newAWS:                     /\`|\~|\!|\@|\#|\$|\%|\^|\&|\*|\(|\)|\+|\=|\[|\{|\]|\}|\||\\|\'|\<|\,|\.|\>|\?|\/|\""|\;|\:|\s/g
                        };

const oneDay        =   24 * 60 * 60 * 1000;

let uninitialized   = exports.uninitialized         =  (data, callback) => callback (new error.srv (200));

function getStuffAndCounts (data, callback) { // data -> {queryArgs, model, skip, propName, modelName}

    let error =             null,
        stuffReturn =       {},
        done =              () => callback (error, stuffReturn);

    if (!data.skip ) {

        done = _.after (2, done);

        models[data.modelName]
            .where( data.queryArgs )
            .count( function (e, count ) {
                if (e) console.log(e);
                stuffReturn.count = count;
                done();
            });
    }

    data.model.exec(function (e, stuff){
        if (e) console.log(e);
        stuffReturn[data.propName] = stuff;
        done();
    });

};

function getStuffAndCountsEarlierThan (data, callback) { // data -> {queryArgs, model, earlierThan, propName, modelName}

    let error =             null,
        stuffReturn =       {},
        done =              () => callback (error, stuffReturn);

    if (!data.earlierThan ) {

        done = _.after (2, done);

        models[data.modelName]
            .where( data.queryArgs )
            .count( function (e, count ) {
                if (e) {
                    console.log(e);
                    error = e;
                }
                stuffReturn.count = count || 0;
                done();
            });
    } else _.each (data.earlierThan, (value, key) => data.model.where(key).lt(value) );

    data.model.exec(function (e, stuff){
        if (e) {
            console.log(e);
            error = e;
        }
        stuffReturn[data.propName] = stuff || [];
        done();
    });

};

function getStuffAndCountsEarlierThanPromise (data) { // data -> {queryArgs, model, earlierThan, propName, modelName}

    return new Promise (function (resolve, reject) {

        let error =             null,
            stuffReturn =       {},
            done =              () => {
                                    if (error) return reject(error);
                                    else return resolve (stuffReturn);
                                };

        if (!data.earlierThan ) {

            done = _.after (2, done);

            models[data.modelName]
                .where( data.queryArgs )
                .count()
                .then (count => {
                    stuffReturn.count = count || 0;
                    done();
                })
                .catch (e => {
                    console.log(e);
                    error = e;
                    done();
                });

        } else _.each (data.earlierThan, (value, key) => data.model.where(key).lt(value));

        return data.model.exec()
            .then(stuff => {
                stuffReturn[data.propName] = stuff || [];
                return done();
            })
            .catch(e => {
                console.log(e);
                error = e;
                return done();
            });
    });
};

function getWeekNumber () {
    let target = new Date();
    let dayNr = (target.getDay() + 6) % 7;
    target.setDate(target.getDate() - dayNr + 3);
    let firstThursday = target.valueOf();
    target.setMonth(0, 1);
    if (target.getDay() !== 4) {
        target.setMonth(0, 1 + ((4 - target.getDay()) + 7) % 7);
    }
    return 1 + Math.ceil((firstThursday - target) / 604800000);

}

function getDayNumber () {
    return (new Date()).getDay();
}

function getDayHourNumber () {
    let now = new Date();
    return { day: now.getDay(), hour: now.getHours() };
}

let getDateStringToday = exports.getDateStringToday = function () {
    let now = new Date();
    return new Date(now.getFullYear(), now.getMonth(), now.getDate());
}

let getDateStringTomorrow = exports.getDateStringTomorrow = function () {
    let tomorrow = new Date(Date.now() + oneDay);
    return new Date(tomorrow.getFullYear(), tomorrow.getMonth(), tomorrow.getDate());
}

function escape (text) {
  if (_.isString(text)) return text.replace(escapeRegEx.allSpecial, "\\$&");
  else return text;
};

function escapeAWSCS (text) {
  if (_.isString(text)) return text.replace(escapeRegEx.blackSlashAndApostrophe, "\\$&");
  else return text;
};

function callbackPrintError (e) {
    if (e) {
        logger.error ("Error: " + e);
        logger.error (new error.srv (200));
    }
}

//// Public ---------------------------------------------------------------------------------------

exports.callbackPrintError =                callbackPrintError;
exports.database =                          database;
exports.error =                             error;
exports.escape =                            escape;
exports.escapeAWSCS =                       escapeAWSCS;
exports.getStuffAndCounts =                 getStuffAndCounts;
exports.getStuffAndCountsEarlierThan =      getStuffAndCountsEarlierThan;
exports.getStuffAndCountsEarlierThanPromise = getStuffAndCountsEarlierThanPromise;
exports.http =                              http;
exports.logger =                            logger;
exports.transact =                          transact;
exports.getWeekNumber =                     getWeekNumber;
exports.getDayNumber =                      getDayNumber;
exports.getDayHourNumber =                  getDayHourNumber;