"use strict";

let underScore      =   require('underscore');

module.exports = {

	regex: {
		alphanumeric            :   /^[a-zA-Z0-9 ]*$/,
	    alphanumericPeriod      :   /^[a-zA-Z0-9 .]*$/,
	    alphanumericNoSpace     :   /^[a-zA-Z0-9]*$/,
	    alphabet                :   /^[A-z ]*$/,
        alphabetNoSpace         :   /^[A-z]*$/,
        numeric 				: 	/^([0-9-])+$/,
        numericFloat 			: 	/^([0-9-.])+$/,
        numericPositive    		:   /^([0-9])+$/,
		email                   :   /^([A-Za-z0-9_\+\-\.])+\@([A-Za-z0-9_\-\.])+\.([A-Za-z]{2,10})$/,
	    
	    majorCreditCards        :   /^(?:4[0-9]{12}(?:[0-9]{3})?|5[1-5][0-9]{14}|6011[0-9]{12}|3(?:0[0-5]|[68][0-9])[0-9]{11}|3[47][0-9]{13})$/,
	    americanExpress         :   /^(3[47][0-9]{13})*$/,
	    discover                :   /^6011-?\d{4}-?\d{4}-?\d{4}$/,
	    masterCard              :   /^(5[1-5][0-9]{14})*$/,
	    visa                    :   /^(4[0-9]{12}(?:[0-9]{3})?)*$/,	    

	    strongPassword          :   /^(?:^.{8,}$)((?:.*[A-Za-z0-9])(?:.*[A-Z])(?:.*[a-z]))^.*$/,
        mongooseObjectId        :   /^[0-9a-fA-F]{24}$/,
        username 				:	/^[a-z0-9_-]{3,16}$/,
		password 				: 	/^.*(?=.{6,32})(?=.*d)(?=.*[A-Z])(?=.*[a-z])(?=.*[!@#$%^&*? ]).*$/,
		url 					:  	/^(https?:\/\/)?([\da-z\.-]+)\.([a-z\.]{2,6})([\/\w \.-]*)*\/?$/,
		ipaddress				: 	/^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/
	},

	functions: {
		array:                 	value => underScore.isArray(value),
		boolean:               	value => underScore.isBoolean (value),
		booleanStringTrueFalse: value => (value === 'true' || value === 'false'),
		booleanStringZeroOne:   value => (value === '0' || value === '1'),
		booleanString: 			function (value) { return (this.booleanStringTrueFalse(value) || this.booleanStringZeroOne(value)); },
		date: 					value => { let d = new Date (value); return d.getTime() === d.getTime(); },
		notEmpty:              	value => !underScore.isEmpty (value),
		//numberLessThan2Days:   	function (value) { return ( value <= config.times.twoDays ); },
		number:        			value => (underScore.isNumber(value) && !underScore.isNaN(value) && value === parseInt(value)),
		positiveNumber:        	value => (underScore.isNumber(value) && !underScore.isNaN(value) && value === parseInt(value) && value >= 0),
		positiveFloat:        	value => (underScore.isNumber(value) && !underScore.isNaN(value) && value >= 0),
		
		string:                	value => underScore.isString(value),
		stringLength64:         value => value.length === 64,
		
		nonEmptyString: 		function (value) { return this.string(value) && this.notEmpty (value); },
		contentTypes: 			function (value) { return this.meetupObject (value) || this.string(value); }
	},
	
	objects: {
		
		file: {
			//name: 		[ { test: 'nonEmptyString', required: false } ],
			path: 		[ { test: 'nonEmptyString', required: true } ],
			size: 		[ { test: 'positiveNumber', required: true } ],
			path: 		[ { test: 'nonEmptyString', required: false } ],
		},

		cashOffered: {
			amount: 	[ { test: 'numericPositive', required: true } ],
			currency: 	[ { test: 'nonEmptyString', required: true } ]
		},
		location: {
			name: 		[ { test: 'string', required: true } ],
			coords: 	[ { test: 'coords', required: true } ]
		},
		meetup: {
			coords: 	[ { test: 'numericFloatArray', required: true } ],
			name: 		[ { test: 'string', required: false } ],
			address1: 	[ { test: 'string', required: false } ],
			address2: 	[ { test: 'string', required: false } ],
			city: 		[ { test: 'string', required: false } ],
			state: 		[ { test: 'string', required: false } ],
			postal: 	[ { test: 'string', required: false } ],
			country: 	[ { test: 'string', required: false } ],
			dateTime: 	[ { test: 'date', required: true } ]
		}
	},

	stringSets: {
        msgTypeSet:         	['text', 'image', 'meetup'],
        channelSet: 			['direct', 'offer', 'item'],
        platformSet: 			['ios', 'android'],
        transactVendorSet: 		['paypal']
    },

    arrays: ['email', 'mongooseObjectId', 'string', 'fileObject', 'number', 'numericFloat'],
	
	rules: {
        _id:            		['mongooseObjectId'],
        _ids:           		['mongooseObjectIdArray'],
        _item:            		['mongooseObjectId'],
        _items:            		['mongooseObjectIdArray'],
        _itemsOffered:     		['mongooseObjectIdArray'],
        _itemsWithdrawn:   		['mongooseObjectIdArray'],
		_offer:            		['mongooseObjectId'],
        _offers:   				['mongooseObjectIdArray'],

    	acceptTrade: 			['string'],
    	cashFirm: 				['booleanString'],
    	cashOffered: 			['cashOfferedObject'],
    	channel: 				['nonEmptyString', 'channelSet'],
    	content: 				['contentTypes'],
    	coord: 					['numericFloatArray'],
    	description:			['nonEmptyString'],
    	deviceToken: 			['nonEmptyString'],
    	email: 					['email'],
        emails:         		['emailArray'],
        facebookAccessToken: 	['nonEmptyString'],
        facebookRefreshToken: 	['nonEmptyString'],
        facebookAuthHash: 		['nonEmptyString'],
        file: 					['fileObject'],
        files: 					['fileObjectArray'],
    	interests:				['stringArray'],
    	itemString:				['nonEmptyString'],
    	laterThan: 				['nonEmptyString', 'date'],
    	location: 				['numberArray'],
    	meetup: 				['meetupObject'],
    	msgType: 				['nonEmptyString', 'msgTypeSet'],
    	name: 					['nonEmptyString'],
        newPassword:    		['nonEmptyString'],
        password: 				['nonEmptyString'],
        platform: 				['nonEmptyString', 'platformSet'],
        price: 					['numericPositive'],
        
        returnProfile: 			['string'],

        skip: 					['numericPositive'],
        limit: 					['numericPositive'],
        tags: 					['nonEmptyString'],
        transactVendor: 		['nonEmptyString', 'transactVendorSet'],
        username: 				['username']
    }

};

/*****************************************************************/

if (process.env.testing === "true") {
	
	module.exports.objects.test1 = {
		name: 		[ { test: 'nonEmptyString', required: true } ],
		email: 		[ { test: 'email', required: true } ],
	};
	module.exports.objects.test = {
		name: 		[ { test: 'nonEmptyString', required: true } ],
		email: 		[ { test: 'email', required: false } ],
		pals: 		[ { test: 'emailArray', required: true } ],
		sideKick: 	[ { test: 'test1Object', required: true } ]
	}	;
	module.exports.stringSets.productSet = ['test1','test2'];
}