process.env.testing = "true";

var assert      =   require("assert"),
    should      =   require('should'),
	validator   =   require('../index.js');

validator.init (require('../config'));

describe('#positiveNumber()', function () {
    it('should validate a positive number true', function (done) {
        var result = validator.positiveNumber(9000);
        should.exist(result); result.should.equal(true);
        done();
    });


    it('should validate a positive number true', function (done) {
        var result = validator.positiveNumber(+9000);
        should.exist(result); result.should.equal(true);
        done();
    });
	
	it('should validate a positive number true', function (done) {
        var result = validator.positiveNumber(90.00);
        should.exist(result); result.should.equal(true);
        done();
    });


	

    it('should validate a floating point false', function (done) {
        var result = validator.positiveNumber(9000/34);
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate a negative number false', function (done) {
        var result = validator.positiveNumber(-9000);
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate a string number as false', function (done) {
        var result = validator.positiveNumber('9000');
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate a string characters false', function (done) {
        var result = validator.positiveNumber('abe');
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate an empty string false', function (done) {
        var result = validator.positiveNumber('');
        should.exist(result); result.should.equal(false);
        done();
    });

});

describe('#date()', function () { 
    it('should validate a date string', function (done) {
        validator.date('2015-01-15T02:31:19.000Z').should.equal(true);
        validator.string('283 Adelaide St W\nToronto‎ ON‎ M5H 1X9\nCanada').should.equal(true);
        validator.numericFloatArray(["-79.39019999999999","43.647807"]).should.equal(true);
        done();
    });
});

describe.skip('#boolean()', function () {
    it('should validate a boolean as true', function (done) {
        var result = validator.boolean(true);
        should.exist(result); result.should.equal(true);
        done();
    });

    it('should validate a boolean as true', function (done) {
        var result = validator.boolean(false);
        should.exist(result); result.should.equal(true);
        done();
    });


    it('should validate a string as false', function (done) {
        var result = validator.boolean('true');
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate numbers as false', function (done) {
        var result = validator.boolean(0);
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate arrays as false', function (done) {
        var result = validator.boolean([true]);
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate objects as false', function (done) {
        var result = validator.boolean({bool:false});
        should.exist(result); result.should.equal(false);
        done();
    });

});



describe('#BooleanString()', function () {
    it('should validate a Boolean String', function (done) {

        validator.booleanStringTrueFalse('true').should.equal(true);
        validator.booleanStringTrueFalse('false').should.equal(true);
        validator.booleanStringTrueFalse('fals').should.equal(false);
        validator.booleanStringTrueFalse({}).should.equal(false);
        validator.booleanStringTrueFalse([]).should.equal(false);
        validator.booleanStringTrueFalse(true).should.equal(false);
        validator.booleanStringTrueFalse(false).should.equal(false);

        validator.booleanStringZeroOne('0').should.equal(true);
        validator.booleanStringZeroOne('1').should.equal(true);
        validator.booleanStringZeroOne(0).should.equal(false);
        validator.booleanStringZeroOne(1).should.equal(false);
        validator.booleanStringZeroOne(2).should.equal(false);
        validator.booleanStringZeroOne(9).should.equal(false);
        validator.booleanStringZeroOne(-1).should.equal(false);
        validator.booleanStringZeroOne({}).should.equal(false);
        validator.booleanStringZeroOne([]).should.equal(false);

        validator.booleanString('true').should.equal(true);
        validator.booleanString('false').should.equal(true);
        validator.booleanString('0').should.equal(true);
        validator.booleanString('1').should.equal(true);
        validator.booleanString(true).should.equal(false);
        validator.booleanString(false).should.equal(false);
        validator.booleanString({}).should.equal(false);
        validator.booleanString([]).should.equal(false);
        validator.booleanString('sdfjhsk').should.equal(false);

        done();
    });
});

describe('#notEmpty()', function () {
    
    it('should validate a string as true', function (done) {
        var result = validator.notEmpty("blah");
        should.exist(result); result.should.equal(true);
        done();
    });

    it('should validate an object as true', function (done) {
        var result = validator.notEmpty({text:"blah"});
        should.exist(result); result.should.equal(true);
        done();
    });

    it('should validate an array as true', function (done) {
        var result = validator.notEmpty(["something"]);
        should.exist(result); result.should.equal(true);
        done();
    });

    it('should validate an empty string as false', function (done) {
        var result = validator.notEmpty("");
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate an empty object as false', function (done) {
        var result = validator.notEmpty({});
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate an empty array as false', function (done) {
        var result = validator.notEmpty([]);
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate null as false', function (done) {
        var result = validator.notEmpty(null);
        should.exist(result); result.should.equal(false);
        done();
    });
});

describe('#mongooseObjectId()', function () {
    
    it('should validate a valid mongooseObjectId as true', function (done) {
        var result = validator.mongooseObjectId("53ab40fbbdc3c37803ae44b8");
        should.exist(result); result.should.equal(true);
        done();
    });

    it('should validate a random string as false', function (done) {
        var result = validator.mongooseObjectId("sdfsdfs");
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate a short hex string as false', function (done) {
        var result = validator.mongooseObjectId("a34b");
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate a number string ', function (done) {
        var result = validator.mongooseObjectId("234234");
        should.exist(result); result.should.equal(false);
        done();
    });
});

describe('#mongooseObjectIdArray()', function () {
    
    it('should validate a valid mongooseObjectIdArray as true', function (done) {
        var result = validator.mongooseObjectIdArray(["53ab40fbbdc3c37803ae44b8"]);
        should.exist(result); result.should.equal(true);
        done();
    });

    it('should validate a valid mongooseObjectIdArray as true', function (done) {
        var result = validator.mongooseObjectIdArray( ["53ab40fbbdc3c37803ae44b8", "53ab41fbbdc3c47803ae44b8"] );
        should.exist(result); result.should.equal(true);
        done();
    });

    it('should validate an invalid mongooseObjectIdArray as false', function (done) {
        var result = validator.mongooseObjectIdArray( ["53ab40fbbdc3c37803ae44b8", "34"] );
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate a random string as false', function (done) {
        var result = validator.mongooseObjectIdArray("sdfsdfs");
        should.exist(result); result.should.equal(false);
        done();
    });

    it('should validate an empty array as false', function (done) {
        var result = validator.mongooseObjectIdArray([]);
        should.exist(result); result.should.equal(false);
        done();
    });
});


describe('#testObject()', function () { 
    it('should validate a testObject as true', function (done) {
        var result = validator.testObject ({
            name: "HalJordan", 
            email: "hal.jordan@ooah.net", 
            pals: ['hal.jordan@ooah.net', 'john.stewart@ooah.net'],
            sideKick : {
            	name: "JohnStewart",
            	email: "john.stewart@ooah.net"
            }

        });
        should.exist(result); result.should.equal(true);
        done();
    });
});

describe('#productSet()', function () {
    
    it('should validate a product string', function (done) {
        var result = validator.productSet("test1");
        should.exist(result); result.should.equal(true);
        done();
    });

    it('should validate a product string', function (done) {
        var result = validator.productSet("test2");
        should.exist(result); result.should.equal(true);
        done();
    });

});