process.env.testing = "true";

var assert      =   require("assert"),
    should      =   require('should'),
    underScore  =   require('underscore'),
	validator   =   require('../index.js'),
    data        =   {
                        emails: require('../data/emails.js')
                    };

validator.init (require('../config'));

describe('#email()', function () {
    it('should validate an email address', function (done) {
        validator.email('someone@somewhere.com').should.equal(true);
        validator.email('someone@somewhere.complement').should.equal(true);
        validator.email('someone@somewhere.co').should.equal(true);
        validator.email('some.one+else@somewhere.co').should.equal(true);
        validator.email('some.one_else@somewhere.co').should.equal(true);
        validator.email('some.one.else@somewhere.co').should.equal(true);
        validator.email('so+me.one+else@somewhere.co').should.equal(true);
        validator.email('s.o.m.e.o+n+e+else@somewhere.co').should.equal(true);

        validator.email('').should.equal(false);
        validator.email('sdfsdf').should.equal(false);
        validator.email('someone@@somewhere.com').should.equal(false);
        validator.email('someonesomewhere.com').should.equal(false);
        validator.email('someone@somewherecom').should.equal(false);
        validator.email('someone@somewhere.c').should.equal(false);
        validator.email('someone@somewhere.complementary').should.equal(false);
        validator.email('$@somewhere.com').should.equal(false);

        underScore.every(data.emails, function (email) { return validator.email(email.email); }).should.equal(true);

        done();
    });
});


describe('#emailArray()', function () { 
    it('should validate an emailArray', function (done) {
        validator.emailArray(['bruce@wayne.com', 'peter@parker.com']).should.equal(true);
        validator.emailArray([]).should.equal(false);
        validator.emailArray('bruce@wayne.com').should.equal(false);
        validator.emailArray(98776).should.equal(false);
        validator.emailArray({email:'bruce@wayne.com'}).should.equal(false);
        validator.emailArray(['bruce@wayne.com', 'peterparker.com']).should.equal(false);
        done();
    });
});

describe('#alphanumeric()', function () {
    it('should validate an alphanumeric string true', function (done) {
        validator.alphanumeric('Som3thing som3thing dark side').should.equal(true);
        validator.alphanumeric('Something something dark side').should.equal(true);
        validator.alphanumeric('314159').should.equal(true);
        validator.alphanumeric(314159).should.equal(true);
        validator.alphanumeric('').should.equal(true);
        validator.alphanumeric(null).should.equal(true);
        
        validator.alphanumeric('Something something dark side.').should.equal(false);
        validator.alphanumeric('Something something dark side!').should.equal(false);
        validator.alphanumeric({text:"blah"}).should.equal(false);
        validator.alphanumeric(["something."]).should.equal(false);
        
        done();
    });
});


describe('#alphanumericPeriod()', function () {
    it('should validate an alphanumeric string with a period true', function (done) {
        var result = validator.alphanumericPeriod('Something something dark side.');
        should.exist(result); result.should.equal(true);
        done();
    });
});

describe('#alphanumericNoSpace()', function () {
    it('should validate an alphanumeric string with a space as false', function (done) {
        var result = validator.alphanumericNoSpace('Something something dark side');
        should.exist(result); result.should.equal(false);
        done();
    });
});

describe('#alphabet()', function () {
    it('should validate an alphanumeric string with as false', function (done) {
        var result = validator.alphabet('Som3thing something dark side');
        should.exist(result); result.should.equal(false);
        done();
    });
});

describe('#numeric()', function () {
    it('should validate a numeric string', function (done) {
        validator.numeric('0').should.equal(true);
        validator.numeric('12').should.equal(true);
        validator.numeric('123').should.equal(true);
        validator.numeric('1234').should.equal(true);
        validator.numeric('-1234').should.equal(true);
        validator.numeric('12.0').should.equal(false);
        validator.numeric('12.12').should.equal(false);
        validator.numeric('-1.1').should.equal(false);
        validator.numeric('abd').should.equal(false);
        validator.numeric({}).should.equal(false);
        validator.numeric({test:'test'}).should.equal(false);
        validator.numeric([]).should.equal(false);
        validator.numeric(['test']).should.equal(false);
        done();
    });
});

describe('#numericPositive()', function () {
    it('should validate an numeric positive string', function (done) {
        validator.numericPositive('0').should.equal(true);
        validator.numericPositive('12').should.equal(true);
        validator.numericPositive('123').should.equal(true);
        validator.numericPositive('1234').should.equal(true);
        validator.numericPositive('-1234').should.equal(false);
        validator.numericPositive('12.0').should.equal(false);
        validator.numericPositive('12.12').should.equal(false);
        validator.numericPositive('-1.1').should.equal(false);
        validator.numericPositive('abd').should.equal(false);
        validator.numericPositive({}).should.equal(false);
        validator.numericPositive({test:'test'}).should.equal(false);
        validator.numericPositive([]).should.equal(false);
        validator.numericPositive(['test']).should.equal(false);
        done();
    });
});



describe('#positiveNumber()', function () {
    it('should validate a positive number true', function (done) {
        validator.positiveNumber(9000).should.equal(true);
        validator.positiveNumber(+9000).should.equal(true);
        validator.positiveNumber(90.00).should.equal(true);
        
        validator.positiveNumber(9000/34).should.equal(false);
        validator.positiveNumber(-9000).should.equal(false);
        validator.positiveNumber('9000').should.equal(false);
        validator.positiveNumber('abe').should.equal(false);
        validator.positiveNumber('').should.equal(false);

        done();
    });
});


describe('#boolean()', function () {
    it('should validate a boolean as true', function (done) {
        validator.boolean('1').should.equal(true);
        validator.boolean('0').should.equal(true);
        validator.boolean(0).should.equal(true);
        validator.boolean(1).should.equal(true);
        validator.boolean('true').should.equal(false);
        validator.boolean(['0']).should.equal(true);
        validator.boolean({bool:'1'}).should.equal(false);
        done();
    });
});

describe('#notEmpty()', function () {
    
    it('should validate a string as true', function (done) {
        validator.notEmpty("blah").should.equal(true);
        validator.notEmpty({text:"blah"}).should.equal(true);
        validator.notEmpty(["something"]).should.equal(true);
        
        validator.notEmpty("").should.equal(false);
        validator.notEmpty({}).should.equal(false);
        validator.notEmpty([]).should.equal(false);
        validator.notEmpty(null).should.equal(false);
        done();
    });
});



describe ('#isValid()', function(){
    it('should validate input data', function (done) {
        var data = {name: 'Ahsoka Tano', email:"ahsokatano001@gmail.com", password:'vanishhSingle'};
        var properties = ['name', 'email', 'password'];
        var result = validator.isValid(data, properties);
        should.exist(result); result.should.equal(true);
        done();
    });

    it('should validate input data as false', function (done) {
        var data = {email:"ahsokatano001@gmail", password:'vanishhSingle'};
        var properties = ['email', 'password'];
        var result = validator.isValid(data, properties);
        should.exist(result); result.should.equal(false);
        done();
    });

});