process.env.testing = "true";

var assert              =   require("assert"),
    should              =   require('should'),
    
    validator           =   require('../index');

validator.init (require('../config'));

var sanitize            =   validator.sanitize;

describe('#sanitize()', function () {
    var insaneData = { name: "moe", age: 12, description: "Blah de Bah <script>alert('test')</script>", settings: { test: "test<script></script>", number: 12} },
        goodData = { name: "moe", age: 12, description: "Blah de Bah ", settings: { test: "test", number: 12} };

    it('should sanitize objects', function (done) {
        sanitize(insaneData, function (saneData) {
            should.exist(saneData);
            should.exist(saneData.name);
            should.exist(saneData.age);
            should.exist(saneData.description);
            should.exist(saneData.settings);
            should.exist(saneData.settings.test);
            should.exist(saneData.settings.number);
            saneData.name.should.equal(goodData.name);
            saneData.age.should.equal(goodData.age);
            saneData.description.should.equal(goodData.description);
            saneData.settings.test.should.equal(goodData.settings.test);
            saneData.settings.number.should.equal(goodData.settings.number);
            done();
        });
    });

    it('should sanitize objects', function (done) {
        insaneData = { name: "moe", age: 12, description: { test: 123, blah: function () { console.log('test') }, something: "Blah de Bah <script>alert('test')</script>" }, settings: { test: "test<script></script>", number: 12} };
        goodData = { name: "moe", age: 12, description: { test: 123, blah: {}, something: "Blah de Bah " }, settings: { test: "test", number: 12} };

        sanitize(insaneData, function (saneData) {
            should.exist(saneData);
            should.exist(saneData.name);
            should.exist(saneData.age);
            should.exist(saneData.description);
            should.exist(saneData.settings);
            should.exist(saneData.settings.test);
            should.exist(saneData.settings.number);
            saneData.name.should.equal(goodData.name);
            saneData.age.should.equal(goodData.age);
            saneData.description.test.should.equal(goodData.description.test);
            saneData.description.something.should.equal(goodData.description.something);
            saneData.settings.test.should.equal(goodData.settings.test);
            saneData.settings.number.should.equal(goodData.settings.number);
            done();
        });
    });

    it('should sanitize objects with arrays', function (done) {
        insaneData = { name: "moe", age: 12, description: ["test1", "test2"] };
        goodData = { name: "moe", age: 12, description: ["test1", "test2"] };
        sanitize(insaneData, function (saneData) {
            should.exist(saneData);
            should.exist(saneData.name);
            should.exist(saneData.age);
            should.exist(saneData.description);
            
            saneData.name.should.equal(goodData.name);
            saneData.age.should.equal(goodData.age);
            saneData.description[0].should.equal(goodData.description[0]);
            done();
        });
    });

    it('should sanitize objects', function (done) {
        insaneData = 'sdfkjh';
        goodData = 'sdfkjh';
        sanitize(insaneData, function (saneData) {
            should.exist(insaneData);
            saneData.should.equal(goodData);
            done();
        });
    });

    it('should sanitize objects', function (done) {
        insaneData = 'sd<script>fk</script>jh';
        goodData = 'sdjh';
        sanitize(insaneData, function (saneData) {
            should.exist(insaneData);
            saneData.should.equal(goodData);
            done();
        });
    });

    it('should sanitize objects', function (done) {
        insaneData = 123987395873948573498573495873495873405837450;
        goodData = 123987395873948573498573495873495873405837450;
        sanitize(insaneData, function (saneData) {
            should.exist(saneData);
            saneData.should.equal(goodData);
            done();
        });
    });

    it('should sanitize objects', function (done) {
        insaneData = { name: "moe&", age: 12, description: { test: 123, blah: function () { console.log('test') }, something: "Blah de Bah <script>alert('test')</script>" }, settings: { test: "test<script></script>", number: 12} };
        goodData = { name: "moe&", age: 12, description: { test: 123, blah: {}, something: "Blah de Bah " }, settings: { test: "test", number: 12} };

        sanitize (insaneData, function (saneData) {
            should.exist(saneData);
            should.exist(saneData.name);
            should.exist(saneData.age);
            should.exist(saneData.description);
            should.exist(saneData.settings);
            should.exist(saneData.settings.test);
            should.exist(saneData.settings.number);
            saneData.name.should.equal(goodData.name);
            saneData.age.should.equal(goodData.age);
            saneData.description.test.should.equal(goodData.description.test);
            saneData.description.something.should.equal(goodData.description.something);
            saneData.settings.test.should.equal(goodData.settings.test);
            saneData.settings.number.should.equal(goodData.settings.number);
            done();
        });
    });

});